Package: crisprBase
Version: 1.17.0
Date: 2025-10-27
Title: Base functions and classes for CRISPR gRNA design
Authors@R: person("Jean-Philippe", "Fortin", email = "fortin946@gmail.com", role = c("aut", "cre"))
Depends: utils, methods, R (>= 4.1)
Imports: BiocGenerics, Biostrings, GenomicRanges, graphics, IRanges,
        S4Vectors, stringr
Suggests: BiocStyle, knitr, rmarkdown, testthat
biocViews: CRISPR, FunctionalGenomics
Description: Provides S4 classes for general nucleases, CRISPR nucleases,
    CRISPR nickases, and base editors.Several CRISPR-specific genome
    arithmetic functions are implemented to help extract genomic coordinates
    of spacer and protospacer sequences. Commonly-used CRISPR nuclease objects
    are provided that can be readily used in other packages.
    Both DNA- and RNA-targeting nucleases are supported.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
BugReports: https://github.com/crisprVerse/crisprBase/issues
URL: https://github.com/crisprVerse/crisprBase
Collate: 'AllGenerics.R' 'rebase.R' 'Nuclease-class.R'
        'Nickase-class.R' 'CrisprNuclease-class.R'
        'CrisprNickase-class.R' 'BaseEditor-class.R' 'arithmetics.R'
        'data.R' 'utils.R' 'converters.R' 'annotateMismatches.R'
LazyData: true
git_url: https://git.bioconductor.org/packages/crisprBase
git_branch: devel
git_last_commit: f896dbd
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:02:43 UTC; biocbuild
Author: Jean-Philippe Fortin [aut, cre]
Maintainer: Jean-Philippe Fortin <fortin946@gmail.com>
