#Required inkscape to be installed
ink enzymes.pdf --export-plain-svg=enzymes.svg
ink nucleases.pdf --export-plain-svg=nucleases.svg
ink nickases.pdf --export-plain-svg=nickases.svg
ink nucleasesBaseEditor.pdf --export-plain-svg=nucleasesBaseEditor.svg
ink cut_sites.pdf --export-plain-svg=cut_sites.svg
